-- G. McCance 2007
CREATE OR REPLACE
package body fts_filestats
IS
                      
-- private variables   
e_badschema EXCEPTION;
PRAGMA EXCEPTION_INIT(e_badschema, -20103);
e_pk_violation EXCEPTION;
PRAGMA EXCEPTION_INIT(e_pk_violation, -0001);

BODY_VERSION CONSTANT VARCHAR(10) := '1.0.0';


-- private methods
procedure log_error
  (v_file_id in number, v_code number, v_mess in varchar2)
AS
PRAGMA AUTONOMOUS_TRANSACTION;
BEGIN
  execute immediate 'insert into T_FILESTATS_ERR(LOGDATE, file_id, err_code, err_msg) values(SYSTIMESTAMP, :1, :2, SUBSTR(:3, 1, 2048))' 
    using v_file_id, v_code, v_mess;
  COMMIT;
END log_error;
         
 
-- public methods

-- abort on schema version mismatch
PROCEDURE check_schema_version
IS
v_schema_vers VARCHAR(20);
BEGIN
   select MAJOR||'.'||MINOR||'.'||PATCH into v_schema_vers from T_SCHEMA_VERS;
   IF v_schema_vers != VALID_SCHEMA_VERSION THEN
      dbms_output.put_line('ERROR: This script only runs with FTS schema version: '||VALID_SCHEMA_VERSION);
      dbms_output.put_line('       You have version: '||v_schema_vers);
      RAISE_APPLICATION_ERROR(-20103, 'This script only runs with FTS schema version: '||VALID_SCHEMA_VERSION);
   END IF;
      select MAJOR||'.'||MINOR||'.'||PATCH into v_schema_vers from T_FILESTATS_SCHEMA_VERS;
   IF v_schema_vers != FILESTATS_SCHEMA_VERSION THEN
      dbms_output.put_line('ERROR: This script only runs with filestats subschema version: '||FILESTATS_SCHEMA_VERSION);
      dbms_output.put_line('       You have version: '||v_schema_vers);
      RAISE_APPLICATION_ERROR(-20103, 'This script only runs with filestats subschema version: '||FILESTATS_SCHEMA_VERSION);
   END IF;


END check_schema_version;

-- print the version of this code
PROCEDURE printversion
IS
BEGIN
   IF HEADER_VERSION != BODY_VERSION THEN
      dbms_output.put_line('ERROR: There is a mismatch between the header and the body of this package!');
      dbms_output.put_line('       Please reload both the header and the body');
      RAISE_APPLICATION_ERROR(-20103, 'Header / Body version mismatch!');
   END IF;
   dbms_output.put_line('Package fts_filestats - version:' || BODY_VERSION);
END printversion;          


-- log the transfer information: used by the fts_logstats trigger
procedure log_transfer
   ( v_file_id T_FILE.FILE_ID%TYPE,
     v_filesize T_FILE.FILESIZE%TYPE,
     v_num_failures T_FILE.NUM_FAILURES%TYPE,
     v_finish_time T_FILE.FINISH_TIME%TYPE,
     v_job_id T_FILE.JOB_ID%TYPE )
AS
BEGIN

execute immediate ('INSERT INTO T_FILESTATS     (FID, LOGDATE, JOB_ID, FILE_ID, 
                                                 USER_DN, VO_NAME, CHANNEL_NAME,
                                                 SOURCE_SITE, DEST_SITE, FILESIZE,
                                                 NUM_FAILURES, FINISH_TIME) 
                     select FILESTATS_SEQ.NEXTVAL, SYSTIMESTAMP, J.job_id, :1, J.user_dn, J.vo_name, J.channel_name, C.source_site, C.dest_site,
                     :2, nvl(:3,0) as num_failures, :4
                     from t_job J, t_channel C where J.job_id = :5
                     and J.channel_name = C.channel_name') 
                     using v_file_id, v_filesize, v_num_failures, v_finish_time, v_job_id;
EXCEPTION
   WHEN OTHERS THEN
       log_error(v_file_id, SQLCODE, SQLERRM); 
END log_transfer;

                              
END;
/
