-- Filestats schema version 1.0.0
CREATE TABLE T_FILESTATS (
        FID           NUMBER,
        LOGDATE       TIMESTAMP(6) WITH TIME ZONE DEFAULT SYSTIMESTAMP,
        JOB_ID        CHAR(36) NOT NULL,
        FILE_ID       NUMBER(38) NOT NULL,
        USER_DN       VARCHAR2(1024) NOT NULL,
        VO_NAME       VARCHAR2(50),
        CHANNEL_NAME  VARCHAR2(32),
        SOURCE_SITE   VARCHAR2(100) NOT NULL,
        DEST_SITE     VARCHAR2(100) NOT NULL,
        FILESIZE      NUMBER(38),
        NUM_FAILURES  NUMBER,
        FINISH_TIME   TIMESTAMP(6) WITH TIME ZONE);

ALTER TABLE T_FILESTATS ADD ( CONSTRAINT FILESTATS_FID_PK PRIMARY KEY (FID) );

CREATE INDEX FILESTATS_LOG_DATE ON T_FILESTATS (LOGDATE);

CREATE TABLE T_FILESTATS_ERR (
        LOGDATE       TIMESTAMP(6) WITH TIME ZONE DEFAULT SYSTIMESTAMP,
        FILE_ID       NUMBER(38) NOT NULL,
        ERR_CODE      NUMBER NULL,
        ERR_MSG       VARCHAR(2048));

CREATE SEQUENCE FILESTATS_SEQ START WITH 1;

CREATE TRIGGER t_filestats_trigger
AFTER UPDATE OF FILE_STATE ON T_FILE
FOR EACH ROW
WHEN (NEW.FILE_STATE = 'Done')
BEGIN
   fts_filestats.log_transfer(:NEW.FILE_ID, :NEW.FILESIZE, :NEW.NUM_FAILURES, :NEW.FINISH_TIME, :NEW.JOB_ID);
END;
/

CREATE TABLE T_FILESTATS_SCHEMA_VERS (
  major NUMBER(2) NOT NULL,
  minor NUMBER(2) NOT NULL,
  patch NUMBER(2) NOT NULL,
  --
  -- save a state when upgrading the schema
  state VARCHAR2(24)
);

INSERT INTO T_FILESTATS_SCHEMA_VERS (major,minor,patch) VALUES (1,0,0);

